#coding:utf-8
import os
import smbus
import sens
import time
#-------------------------------------------------------------------------------
# Классы-обертки для контроллеров ТЭНа и клапана отбора
# Список исправлений - в конце файла
# OldBean 17.01.23
#-------------------------------------------------------------------------------
class TEH(object): # Класс нагревателя куба
  def __init__(self, addrTEH = 0x04, addrRMS = 0x05, W_220 = 1418.0, bus = smbus.SMBus(1)):
    self._addrTEH = addrTEH # Адрес контроллера ТЭНа
    self._addrRMS = addrRMS # Адрес датчика RMS напряжения сети
    self._W_220 = W_220     # Номинальная мощность ТЭНа (при 220В)
    self._coef = 4840000.0/self._W_220
    self._bus = bus         # Рабочая шина I2C
    self._vs = sens.RMS(self._addrRMS) # Объект - датчик RMS

    if self._addrRMS < 0:
      print "Предупреждение! Датчик RMS не указан. \nУстанавливаемая мощность ТЭНа может не соответствовать реальной!"

  @property
  def W(self):
    teh_pc = self._bus.read_byte(self._addrTEH)*0.01
    if self._addrRMS < 0: # Работаем без датчика RMS
      wrk = 1.0
    else:
      wrk = self._vs.V*0.00454545454545 # V/220
    return teh_pc*self._W_220*wrk*wrk

  @W.setter
  def W(self, val): # val - желаемая мощность ТЭНа в ваттах
    if self._addrRMS < 0: v = 220.0 # Работаем без датчика RMS
    else: v = self._vs.V

    if v < 140.0:
      print "Напряжение в сети < %3d В. Не буду работать." % (v)
      self._bus.write_byte(self._addrTEH, 0)
      return

    if val < 0: wpc = 0
    elif val > self._W_220: wpc = 100
    else:
      wrk = 220.0/v
      wpc = int(((100.0*val)/self._W_220)*wrk*wrk + 0.5)
      if wpc > 100: wpc = 100
      
    # Попытаемся записать байт в ведомое устройство по шине I2C
    tries = 10; error = None
    while tries: # Если будет сбой при записи - сделаем tries попыток
      try: self._bus.write_byte(self._addrTEH, wpc)
      except IOError as e:
        error = e; tries -= 1
        print "tries = " + str(tries) + "; " + str(error)
      else: break # Нормально отправили байт - выходим из цикла
    if not tries: raise error # Сбой устойчивый - бросаем исключение и завязываем с этим делом
#-------------------------------------------------------------------------------
class SD(object): # Класс устройства отбора
  def __init__(self, addr = 0x06, coef = 9.5053486617, bus = smbus.SMBus(1)):
    self._addr = addr       # Адрес устройства на шине
    self._coef = coef       # Коэффициент для пересчета мл/час в % ШИМ
    self._coefi = 1.0/coef  # Обратная величина coef
    self._bus = bus         # Рабочая шина I2C

  @property
  def Q(self):
    return self._bus.read_byte(self._addr)*self._coef

  @Q.setter
  def Q(self, val): # val - расход в мл/час
    if val < 0: qpc = 0
    else: qpc = int(val*self._coefi + 0.5) # qpc - % ШИМ
    if qpc > 100: qpc = 100 # Ограничение на 100%
      
    # Попытаемся записать байт в ведомое устройство по шине I2C
    tries = 10; error = None
    while tries: # Если будет сбой при записи - сделаем tries попыток
      try: self._bus.write_byte(self._addr, qpc)
      except IOError as e:
        error = e; tries -= 1
        print "tries = " + str(tries) + "; " + str(error)
      else: break # Нормально отправили байт - выходим из цикла
    if not tries: raise error # Сбой устойчивый - бросаем исключение и завязываем с этим делом
#-------------------------------------------------------------------------------
if __name__ == "__main__": # Тестирование классов модуля sens
  print("\nКонтроллер ТЭНа")
  try:
    teh = TEH()
    print "%5.1f Вт" % (teh.W)
    teh.W = 600
    print "%5.1f Вт - рабочий режим" % (teh.W)
    time.sleep(10)
    teh.W = 2000
    print "%5.1f Вт - максимальная мощность" % (teh.W)
    time.sleep(10)
    teh.W = 0
    print "%5.1f Вт - выключен" % (teh.W)
  except:
    print("Контроллер ТЭНа или датчик RMS отсутствуют или неисправны")

  print("\nКонтроллер ТЭНа. Работа без датчика RMS")
  try:
    teh = TEH(4, -1) # Датчик RMS отключен
    print "%5.1f Вт" % (teh.W)
    teh.W = 600
    print "%5.1f Вт - рабочий режим" % (teh.W)
    time.sleep(10)
    teh.W = 2000
    print "%5.1f Вт - максимальная мощность" % (teh.W)
    time.sleep(10)
    teh.W = 0
    print "%5.1f Вт - выключен" % (teh.W)
  except:
    print("Контроллер ТЭНа или датчик RMS отсутствуют или неисправны")

  print("\nУстройство отбора")
  try:
    sd = SD()
    print "%6.1f мл/час" % (sd.Q)
    sd.Q = 400.0
    print "%6.1f мл/час - рабочий режим" % (sd.Q)
    time.sleep(20)
    sd.Q = 0.0
    print "%6.1f мл/час - выключен" % (sd.Q)
  except:
    print("Устройство отбора отсутствует или неисправно")
#---------------------------------------------------------------------------------------
# 23.01.2017
# 1. Добавлена обработка исключения IOError при посылке байта контроллерам в сеттеры
#    классов TEH и SD. При возникновении исключения будут сделаны еще попытки (до 10)
#    отправить байт. Если сбой устойчивый (все 10 попыток неудачны) - исключение кидается
#    выше. 
